// # Play


// To get started, boot the server.

s.boot

/*********************************** Buffer ***********************************/

// [Buffer](https://doc.sccode.org/Classes/Buffer.html) is a UGen to store data on
// the audio server.  For our purposes, we will use `Buffer` to store audio data. 
// An audio file is simply a sequence of numbers called samples.  For
// SuperCollider, use audio file formats like .aiff and .wav (i.e., uncompressed
// and lossless files).  Audio files generally contain one or two channels of audio
// data called mono or stereo.  If you want to use an audio file in SuperCollider,
// you need to know ahead of time whether that audio file is mono or stereo.

/**************************************************
 * Step 1: Find the path to an audio file to play *
 **************************************************/

// In order to play an audio file, you first need to know where the audio file
// exists on your computer.  Every file and folder on your computer has a path
// associated with it.  If you are running this code within the SuperCollider IDE
// you can use the shorthand below.  `thisProcess.nowExecutingPath` gives you the
// file path of the current .scd file you are working with.  `.dirname` gives you
// the folder where your file is located and then the operator `+/+` joins the path
// of the folder with the sounds folder that resides in that folder.


// If you are running this in the notebook, then you need to compute this manually.
//  First find the folder that has the notebook and then get the path to the sounds
// folder.


// If on a mac: https://setapp.com/how-to/how-to-find-the-path-of-a-file-in-mac  

// If on a PC:
// 
// 
// 
// ps://www.howtogeek.com/670447/how-to-copy-the-full-path-of-a-file-on-windows-10/
//  


// Note in the SuperCollider IDE, you can also just drag the file or folder onto
// the screen and the path will be rendered automatically.

// #### Run this if in the SuperCollider IDE
~buffer_folder_path = thisProcess.nowExecutingPath.dirname +/+ "sounds"

// #### Set this variable for the sounds folder if in the notebook
~buffer_folder_path = "/Users/andrewdavis/Documents/CS/Wellesley/CS203/lecture_materials/play/code/sounds" // enter your path here

// If the above worked then let's playback some sounds.

/*****************************
 * Step 2: Creating a Buffer *
 *****************************/

// The sounds folder associated with this lecture has three audio files in it:
// poker.wav, hello.wav, and guitar.aiff.  Let's create a buffer to store each one
// of those audio files.

~poker_buf = Buffer.read(s, ~buffer_folder_path +/+ "poker.wav") // stereo

~hello_buf = Buffer.read(s, ~buffer_folder_path +/+ "hello.wav") // stereo

~guitar_buf = Buffer.read(s, ~buffer_folder_path +/+ "guitar.wav") // A mono file

/*********************** Using PlayBuf to play a buffer ***********************/

// The UGen [PlayBuf](https://doc.sccode.org/Classes/PlayBuf.html) is the
// convenient way to play an audio buffer on the server.  `PlayBuf` needs to know
// ahead of time the number of channels the audio file has.  Therefore, there are
// two versions of a SynthDef designed to play an audio buffer: a mono and stereo
// version.  The first argument to `PlayBuf` is the number of channels.  Notice how
// `mono_play` uses 1 and `stereo_play` uses 2.  


// PlayBuf also takes a buffer object to read from and then several additional
// arguments.  Of particular interest are the arguments `rate` which affects the
// speed of playback and `loop` which can loop the audio file indefinitely. 
// `PlayBuf` also accepts a done action as its last argument.  We can set the done
// action to 2 to free the synth automatically when the audio file has finished
// playing.

(
SynthDef(\mono_play, {
  arg outBus = 0, buf, rate = 1, loop = 0, amp = 1;
  var sig = PlayBuf.ar(1, buf, rate: rate, loop: loop, doneAction: 2);
  
  sig = sig ! 2; // convert mono sound to stereo array to hear out of both speakers
  Out.ar(outBus, sig * amp);
}).add;
)

s.plotTree

Synth(\mono_play, [\buf, ~guitar_buf]) // Note I don't need to free because of doneAction

// Stereo version of the SynthDef above.

(
SynthDef(\stereo_play, {
  arg outBus = 0, buf, rate = 1, loop = 0, amp = 1;
  var sig = PlayBuf.ar(2, buf, rate: rate, loop: loop, doneAction: 2); 
  Out.ar(outBus, sig * amp);
}).add;
)

Synth(\stereo_play, [\buf, ~poker_buf]) // Will free when done

Synth(\stereo_play, [\buf, ~hello_buf])

/************************ Changing the rate of playback ************************/

// The playback rate can be changed using the `play` argument in `PlayBuf`.  This
// is a crude way to change the pitch of an audio file or computer-generated sound
// like a sawtooth wave.  Essentially audio samples are skipped or added to account
// for the respective contraction/elongation of the playback.  We'll learn more
// about why the pitch drops or rises when we talk about digital representation.


// Increasing the playback rate raises the pitch and shortens the duration of the
// audio file.  The file sounds more like a chipmunk now.

~poker_synth = Synth(\stereo_play, [\buf, ~poker_buf, \rate, 2]) // double the playback rate

~poker_synth.free

// Decreasing the playback rate drops the pitch and extends the duration of the
// audio file.

~hello_synth = Synth(\stereo_play, [\buf, ~hello_buf, \rate, 0.5]) // halving the playback rate

~hello_synth.free

// You can also playback the audio file in reverse as well if you provide a
// negative rate.  Note here that I have also looped the file as well.

~hello_synth = Synth(\stereo_play, [\buf, ~hello_buf, \rate, -1, \loop, 1]) // halving the playback rate

~hello_synth.free

/************************** Adding Effects to Audio **************************/

// One of the interesting ways to use audio files in SuperCollider is to route the
// output to an audio effect.  Below we will create a SynthDef that creates a
// reverb effect giving the sound more space.

(
SynthDef(\reverb, {
  arg outBus = 0, inBus, mix = 0.5;
  var sig;
  
  sig = In.ar(inBus, 2);
  sig = FreeVerb2.ar(sig[0], sig[1], mix: mix);
  Out.ar(outBus, sig);
}).add;
)

// Open up a plot tree to see the nodes on the server.

s.plotTree

// Create the reverb synth and a bus to connect to it.

(
~reverb_bus = Bus.audio(s, 2);
~reverb = Synth(\reverb, [\inBus, ~reverb_bus]);
)

// Execute the cell below to play the guitar track through the reverb.  You can
// change the mix of the reverb to increase/decrease the amount of reverberated
// sound.  Because the SynthDef `\mono_play` uses a done action of 2, the synth
// will free itself automatically once it is done playing.

~reverb.set(\mix, 0.4); // Change the mix (0 to 1) to increase/decrease reverb

Synth(\mono_play, [\buf, ~guitar_buf, \outBus, ~reverb_bus], ~reverb, \addBefore);

// Free the bus and reverb synth when done.

(
~reverb_bus.free;
~reverb.free;
)

